//
//  Eraser.m
//  ScreenSaver Eraseing screen functions
//
//  Created by pecos on Wed May 30 2001.
//  Copyright (c) 2001 __CompanyName__. All rights reserved.
//

/* erase.c: Erase the screen in various more or less interesting ways.
 * (c) 1997 by Johannes Keukelaar <johannes@nada.kth.se>
 * Permission to use in any way granted. Provided "as is" without expressed
 * or implied warranty. NO WARRANTY, NO EXPRESSION OF SUITABILITY FOR ANY
 * PURPOSE. (I.e.: Use in any way, but at your own risk!)
 */

#import "Eraser.h"
#include <stdlib.h>

#define LOG_DEBUG

@implementation TestEffects (ErasingComponent)

#undef countof
#define countof(x) (sizeof(x)/sizeof(*(x)))

typedef BOOL (*Eraser) (int width, int height);
static BOOL random_lines (int width, int height);
static BOOL venetian (int width, int height);
static BOOL triple_wipe (int width, int height);
static BOOL quad_wipe (int width, int height);
static BOOL circle_wipe (int width, int height);
static BOOL three_circle_wipe (int width, int height);
static BOOL squaretate (int width, int height);
static BOOL fizzle (int width, int height);
static BOOL random_squares (int width, int height);
static BOOL spiral (int width, int height);

static Eraser erasers[] = {
  random_lines,
  venetian,
  triple_wipe,
  quad_wipe,
  circle_wipe,
  three_circle_wipe,
  squaretate,
  fizzle,
  random_squares,
  spiral
};

- (BOOL)eraseScreen;
{
    static int mode = -1;
    BOOL needToContinue;
    static int foo = 0;
    
    if( mode == -1 ) {
        // mode = SSRandomIntBetween(0, countof(erasers)-1 );
        mode = (foo/5) % countof(erasers);
        foo ++;
#ifdef LOG_DEBUG
        switch( mode ) {
            case 0:
                NSLog( @"eraser: random_lines" );
                break;
            case 1:
                NSLog( @"eraser: venetian" );
                break;
            case 2:
                NSLog( @"eraser: triple_wipe" );
                break;
            case 3:
                NSLog( @"eraser: quad_wipe" );
                break;
            case 4:
                NSLog( @"eraser: circle_wipe" );
                break;
            case 5:
                NSLog( @"eraser: three_circle_wipe" );
                break;
            case 6:
                NSLog( @"eraser: squaretate" );
                break;
            case 7:
                NSLog( @"eraser: fizzle" );
                break;
            case 8:
                NSLog( @"eraser: random_squares" );
                break;
            case 9:
                NSLog( @"eraser: spiral" );
                break;
            default:
                NSLog( @"eraser: ERROR, unknown eraser" );
                break;
       }
#endif
    }
    
    
    needToContinue = (*(erasers[mode])) ([self bounds].size.width, [self bounds].size.height);
    
    if( needToContinue == false ) // screen is totally black
        mode = -1;
    
    return needToContinue;
}

static BOOL random_lines ( int width, int height )
{
    static BOOL horiz_p;
    static int max;
    static int *lines = 0;
    static int i;
    int j;
    float oldDefaultLineWidth;
    BOOL flag = false;
    // NSRect rect;
    
    if( lines == 0 ) {
        horiz_p = SSRandomIntBetween(0, 1);
        max = (horiz_p ? height : width);
        lines = (int *) calloc(max, sizeof(*lines));
        flag = true;
    }
    if( lines == 0 ) {
        NSBeep();
        return false;
    }
    
    if( flag ) {
        for (i = 0; i < max; i++)
            lines[i] = i;

        for (i = 0; i < max; i++) {
            int t, r;
            t = lines[i];
            r = SSRandomIntBetween(0, max-1);
            lines[i] = lines[r];
            lines[r] = t;
        }
        i = 0;
    }
    
    [[NSColor blackColor] set];
    
    oldDefaultLineWidth = [NSBezierPath defaultLineWidth];
    [NSBezierPath setDefaultLineWidth:2 ];
    
    j = 0;
    do {
        if (horiz_p)
            [NSBezierPath strokeLineFromPoint:NSMakePoint(0, lines[i])
                                      toPoint:NSMakePoint(width, lines[i]) ];
        else	
            [NSBezierPath strokeLineFromPoint:NSMakePoint(lines[i], 0)
                                      toPoint:NSMakePoint(lines[i], height) ];
        i ++;
        j ++;
    } while ( i < max && j < 16 );
        
    [NSBezierPath setDefaultLineWidth:oldDefaultLineWidth ];
    /*
    if (horiz_p) {
        rect.size = NSMakeSize( width-1, 1 );
        rect.origin = NSMakePoint(0, lines[i]);
    }
    else {
        rect.size = NSMakeSize( 1, height-1 );
        rect.origin = NSMakePoint(lines[i], 0);
    }
    [NSBezierPath fillRect:rect ];
    */
    
    if( i == max ) {
        NSRect rect;
        rect.size = NSMakeSize( width, height );
        rect.origin = NSMakePoint(0, 0);
        free(lines);
        lines = 0;
        [NSBezierPath fillRect:rect];
        return false;
    }
    
    return true;
}


static BOOL venetian ( int width, int height )
{
    static BOOL horiz_p;
    BOOL flip_p;
    static int max;
    static int *lines = 0;
    BOOL flag = false;
    float oldDefaultLineWidth;
    static int i;
    int j;
    
    if( lines == 0 ) {
        horiz_p = SSRandomIntBetween(0, 1);
        flip_p = SSRandomIntBetween(0, 1);
        max = (horiz_p ? height : width);
        lines = (int *) calloc(max, sizeof(*lines));
        flag = true;
    }
    if( lines == 0 ) {
        NSBeep();
        return false;
    }
    
    if( flag ) {
        j = 0;
        for (i = 0; i < max*2; i++) {
            int line = ((i / 16) * 16) - ((i % 16) * 15);
            if (line >= 0 && line < max)
                lines[j++] = (flip_p ? max - line : line);
        }
        i = 0;
    }
    
    [[NSColor blackColor] set];
    
    oldDefaultLineWidth = [NSBezierPath defaultLineWidth];
    [NSBezierPath setDefaultLineWidth:2 ];
    
    j = 0;
    do {
        if (horiz_p)
            [NSBezierPath strokeLineFromPoint:NSMakePoint(0, lines[i])
                                      toPoint:NSMakePoint(width, lines[i]) ];
        else
            [NSBezierPath strokeLineFromPoint:NSMakePoint(lines[i], 0)
                                      toPoint:NSMakePoint(lines[i], height) ];
        i ++;
        j ++;
    } while ( i < max && j < 16 );
    
    [NSBezierPath setDefaultLineWidth:oldDefaultLineWidth ];
    
    if( i == max ) {
        NSRect rect;
        rect.size = NSMakeSize( width, height );
        rect.origin = NSMakePoint(0, 0);
        free(lines);
        lines = 0;
        [NSBezierPath fillRect:rect];
        return false;
    }
    
    return true;
}

static BOOL triple_wipe ( int width, int height )
{
    static BOOL flip_x;
    static BOOL flip_y;
    int max = width + (height / 2);
    static int *lines = 0;
    static int i;
    int j;
    BOOL flag = false;
    float oldDefaultLineWidth;
    
    
    if( lines == 0 ) {
        flip_x = SSRandomIntBetween(0, 1);
        flip_y = SSRandomIntBetween(0, 1);
        lines = (int *)calloc(max, sizeof(*lines));
        flag = true;
    }
    if( lines == 0 ) {
        NSBeep();
        return false;
    }
    
    if( flag ) {
        for(i = 0; i < width/2; i++)
            lines[i] = i*2+height;
        for(i = 0; i < height/2; i++)
            lines[i+width/2] = i*2;
        for(i = 0; i < width/2; i++)
            lines[i+width/2+height/2] = width-i*2-(width%2?0:1)+height;
        i = 0;
    }
    
    [[NSColor blackColor] set];
    
    oldDefaultLineWidth = [NSBezierPath defaultLineWidth];
    [NSBezierPath setDefaultLineWidth:1.5 ];
    
    j = 0;
    do { 
        int x, y, x2, y2;
        if (lines[i] < height)
            x = 0, y = lines[i], x2 = width, y2 = y;
        else
            x = lines[i]-height, y = 0, x2 = x, y2 = height;

        if (flip_x)
            x = width-x, x2 = width-x2;
        if (flip_y)
            y = height-y, y2 = height-y2;
    
        [NSBezierPath strokeLineFromPoint:NSMakePoint(x, y)
                                  toPoint:NSMakePoint(x2, y2) ];

        i++;
        j++;
    } while ( i < max && j < 16 );

    [NSBezierPath setDefaultLineWidth:oldDefaultLineWidth ];

    if( i == max ) {
        NSRect rect;
        rect.size = NSMakeSize( width, height );
        rect.origin = NSMakePoint(0, 0);
        free(lines);
        lines = 0;
        [NSBezierPath fillRect:rect];
        return false;
    }
    
    return true;
}

static BOOL quad_wipe ( int width, int height )
{
    static BOOL flip_x;
    static BOOL flip_y;
    int max = width + height;
    static int *lines = 0;
    static int i;
    BOOL flag = false;
    float oldDefaultLineWidth;
    int j;
    
    if( lines == 0 ) {
        flip_x = SSRandomIntBetween(0, 1);
        flip_y = SSRandomIntBetween(0, 1);
        lines = (int *)calloc(max, sizeof(*lines));
        flag = true;
    }
    if( lines == 0 ) {
        NSBeep();
        return false;
    }
    
    if( flag ) {
        for (i = 0; i < max/4; i++) {
            lines[i*4]   = i*2;
            lines[i*4+1] = height-i*2-(height%2?0:1);
            lines[i*4+2] = height+i*2;
            lines[i*4+3] = height+width-i*2-(width%2?0:1);
        }
        i = 0;
    }

    [[NSColor blackColor] set];
    
    oldDefaultLineWidth = [NSBezierPath defaultLineWidth];
    [NSBezierPath setDefaultLineWidth:1.5 ];
    
    j = 0;
    do { 
        int x, y, x2, y2;
        if (lines[i] < height)
            x = 0, y = lines[i], x2 = width, y2 = y;
        else
            x = lines[i]-height, y = 0, x2 = x, y2 = height;

        if (flip_x)
            x = width-x, x2 = width-x2;
        if (flip_y)
            y = height-y, y2 = height-y2;

        [NSBezierPath strokeLineFromPoint:NSMakePoint(x, y)
                                  toPoint:NSMakePoint(x2, y2) ];

        i ++;
        j ++;
    } while ( i < max && j < 16 );

    [NSBezierPath setDefaultLineWidth:oldDefaultLineWidth ];

    if( i == max ) {
        NSRect rect;
        rect.size = NSMakeSize( width, height );
        rect.origin = NSMakePoint(0, 0);
        free(lines);
        lines = 0;
        [NSBezierPath fillRect:rect];
        return false;
    }
    
    return true;
}

static BOOL circle_wipe ( int width, int height )
{
    double full = 2 * M_PI;
    static double inc;
    static double start = -1;
    int rad = (width > height ? width : height);
    double rad_c = rad/cos(inc/2.0);
    static int i;
    static NSPoint from;
    static NSPoint center;
    NSPoint middle;
    NSPoint to;
    NSBezierPath *theArc;
    
    if( start == -1 ) {
        start = SSRandomFloatBetween( 0.0, full );
        i = 0;
        inc = full / 64;
        if( SSRandomIntBetween(0, 1) )
            inc *= -1;
            
        center = NSMakePoint(width/2, height/2);
        from = NSMakePoint(width/2 + rad * cos(start),
                           height/2  + rad * sin(start) );
    }
    
    middle = NSMakePoint(width/2 + rad_c * cos(start+(i+0.5)*inc),
                         height/2  + rad_c * sin(start+(i+0.5)*inc) );

    to = NSMakePoint(width/2 + rad * cos(start+(i+1)*inc),
                     height/2  + rad * sin(start+(i+1)*inc) );

    
    [[NSColor blackColor] set];
    theArc = [NSBezierPath bezierPath];
    [theArc moveToPoint:center];
    [theArc lineToPoint:from];
    [theArc curveToPoint:to controlPoint1:middle controlPoint2:middle];
    [theArc closePath];
    [theArc setLineWidth:2.0];
    [theArc fill];
    [theArc stroke];
    
    i++;
    from = to;
    
    if( i == 64 ) {
        start = -1;
        return false;
    }
    
    return true;
}

static BOOL three_circle_wipe ( int width, int height )
{
    static double i;
    double full = 2*M_PI;
    double q = full / 6;
    double q2 = full / 3;
    double inc = full / 180;
    static double start = -1;
    static double angle;
    int rad = (width > height ? width : height);
    double rad_c;
    static NSPoint center;
    NSPoint from;
    NSPoint middle;
    NSPoint to;
    NSBezierPath *theArc_1;
    NSBezierPath *theArc_2;
    NSBezierPath *theArc_3;
    
    if( start == -1 ) {
        start = SSRandomFloatBetween(0, q2);
        i = 0;
        center = NSMakePoint(width/2, height/2);
        angle = inc;
    }
    
    [[NSColor blackColor] set];
    
    rad_c = rad/cos(angle);
    
    from = NSMakePoint(width/2 + rad * cos(start+angle),
                       height/2  + rad * sin(start+angle) );
    middle = NSMakePoint(width/2 + rad_c * cos(start),
                         height/2  + rad_c * sin(start) );
    to = NSMakePoint(width/2 + rad * cos(start-angle),
                     height/2  + rad * sin(start-angle) );
                     
    theArc_1 = [NSBezierPath bezierPath];
    [theArc_1 moveToPoint:center];
    [theArc_1 lineToPoint:from];
    [theArc_1 curveToPoint:to controlPoint1:middle controlPoint2:middle];
    [theArc_1 closePath];
    [theArc_1 setLineWidth:2.0];
    
    from = NSMakePoint(width/2 + rad * cos(start-q2+angle),
                       height/2  + rad * sin(start-q2+angle) );
    middle = NSMakePoint(width/2 + rad_c * cos(start-q2),
                         height/2  + rad_c * sin(start-q2) );
    to = NSMakePoint(width/2 + rad * cos(start-q2-angle),
                     height/2  + rad * sin(start-q2-angle) );
                     
    theArc_2 = [NSBezierPath bezierPath];
    [theArc_2 moveToPoint:center];
    [theArc_2 lineToPoint:from];
    [theArc_2 curveToPoint:to controlPoint1:middle controlPoint2:middle];
    [theArc_2 closePath];
    [theArc_2 setLineWidth:2.0];
    
    from = NSMakePoint(width/2 + rad * cos(start+q2+angle),
                       height/2  + rad * sin(start+q2+angle) );
    middle = NSMakePoint(width/2 + rad_c * cos(start+q2),
                         height/2  + rad_c * sin(start+q2) );
    to = NSMakePoint(width/2 + rad * cos(start+q2-angle),
                     height/2  + rad * sin(start+q2-angle) );
                     
    theArc_3 = [NSBezierPath bezierPath];
    [theArc_3 moveToPoint:center];
    [theArc_3 lineToPoint:from];
    [theArc_3 curveToPoint:to controlPoint1:middle controlPoint2:middle];
    [theArc_3 closePath];
    [theArc_3 setLineWidth:2.0];
    
    
    [theArc_1 fill];
    [theArc_1 stroke];
    [theArc_2 fill];
    [theArc_2 stroke];
    [theArc_3 fill];
    [theArc_3 stroke];
    
    if( angle >= q ) {
        start = -1;
        return false;
    }
    
    angle += inc;
    return true;
}


static BOOL squaretate ( int width, int height )
{
    int steps = (((width > height ? width : height) * 2) / 20);
    static int i = -1;
    int j;
    static BOOL flip;
    NSPoint points [3];
    NSBezierPath *thePath[4];
    
    if( i == -1 ) {
        i = 0;
        flip = SSRandomIntBetween(0, 1);
    }
    
#define DO_FLIP() \
      if (flip) { \
	points[0].x = width-points[0].x; \
	points[1].x = width-points[1].x; \
        points[2].x = width-points[2].x; }
        thePath[0] = [NSBezierPath bezierPath];
        [thePath[0] moveToPoint:points[0]];
        [thePath[0] lineToPoint:points[1]];
        [thePath[0] lineToPoint:points[2]];
        [thePath[0] closePath];
        [thePath[0] setLineWidth:2.0];
        [thePath[0] fill];
        [thePath[0] stroke];

    [[NSColor blackColor] set];
    
    j = 0;
    while ( i < steps && j < 1 ) {
	points[0] = NSMakePoint( 0, 0 );
	points[1] = NSMakePoint( width, 0 );
	points[2] = NSMakePoint( 0, points[0].y + ((i * height) / steps) );
	DO_FLIP();
        thePath[0] = [NSBezierPath bezierPath];
        [thePath[0] moveToPoint:points[0]];
        [thePath[0] lineToPoint:points[1]];
        [thePath[0] lineToPoint:points[2]];
        [thePath[0] closePath];
        [thePath[0] setLineWidth:2.0];
        [thePath[0] fill];
        [thePath[0] stroke];

	points[0] = NSMakePoint( 0, 0 );
	points[1] = NSMakePoint( 0, height );
	points[2] = NSMakePoint( ((i * width) / steps), height );
        DO_FLIP();
        thePath[1] = [NSBezierPath bezierPath];
        [thePath[1] moveToPoint:points[0]];
        [thePath[1] lineToPoint:points[1]];
        [thePath[1] lineToPoint:points[2]];
        [thePath[1] closePath];
        [thePath[1] setLineWidth:2.0];
        [thePath[1] fill];
        [thePath[1] stroke];
    
	points[0] = NSMakePoint( width, height );
	points[1] = NSMakePoint( 0, height );
	points[2] = NSMakePoint( width, height - ((i * height) / steps) );
        DO_FLIP();
        thePath[2] = [NSBezierPath bezierPath];
        [thePath[2] moveToPoint:points[0]];
        [thePath[2] lineToPoint:points[1]];
        [thePath[2] lineToPoint:points[2]];
        [thePath[2] closePath];
        [thePath[2] setLineWidth:2.0];
        [thePath[2] fill];
        [thePath[2] stroke];
    
	points[0] = NSMakePoint( width, height );
	points[1] = NSMakePoint( width, 0 );
	points[2] = NSMakePoint( width - ((i * width) / steps), 0 );
        DO_FLIP();
        thePath[3] = [NSBezierPath bezierPath];
        [thePath[3] moveToPoint:points[0]];
        [thePath[3] lineToPoint:points[1]];
        [thePath[3] lineToPoint:points[2]];
        [thePath[3] closePath];
        [thePath[3] setLineWidth:2.0];
        [thePath[3] fill];
        [thePath[3] stroke];
        
        i ++;
        j ++;
    }
#undef DO_FLIP
    
    if( i == steps ) {
        NSRect rect;
        rect.size = NSMakeSize( width, height );
        rect.origin = NSMakePoint(0, 0);
        [NSBezierPath fillRect:rect];
        i = -1;
        return false;
    }
    
    return true;
    
}


/* from Frederick Roeber <roeber@netscape.com> */
static BOOL fizzle ( int width, int height )
{
  /* These dimensions must be prime numbers.  They should be roughly the
     square root of the width and height. */
# define BX 31
# define BY 31
# define SIZE (BX*BY)
    
    static BOOL running = false;
    static int array[SIZE];
    static int i;
    int j;
    static int *skews_x;
    static int *skews_y;
    static int nx;
    static int ny;
    NSRect theRect;
    BOOL flag = false;
    
    [[NSColor blackColor] set];
    theRect.size = NSMakeSize( 2, 2 );

    /* Distribute the numbers [0,SIZE) randomly in the array */
    if( running == false ) {
        int indices[SIZE];

        for( i = 0; i < SIZE; i++ ) {
            array[i] = -1;
            indices[i] = i;
        } 

        for( i = 0; i < SIZE; i++ ) {
            j = SSRandomIntBetween(0, SIZE-i-1);
            array[indices[j]] = i;
            indices[j] = indices[SIZE-i-1];
        }
        
        /* nx, ny are the number of cells across and down, rounded up */
        nx = width  / BX + (0 == (width %BX) ? 0 : 1);
        ny = height / BY + (0 == (height%BY) ? 0 : 1);
        skews_x = (int *)malloc(sizeof(int) * (nx*ny));
        skews_y = (int *)malloc(sizeof(int) * (nx*ny));
        if( skews_x && skews_y ) {
            for( i = 0; i < nx; i++ ) {
                for( j = 0; j < ny; j++ ) {
                    skews_x[j * nx + i] = SSRandomIntBetween(0, BX - 1);
                    skews_y[j * nx + i] = SSRandomIntBetween(0, BY - 1);
                }
            }
        }
        
        running = true;
        i = 0;
    }
    
    
# define SKEWX(cx, cy) (((int *)0 == skews_x)?0:skews_x[cy*nx + cx])
# define SKEWY(cx, cy) (((int *)0 == skews_y)?0:skews_y[cy*nx + cx])
    
    for( ; (i < SIZE) && (!flag); i++) {
        int x = array[i] % BX;
        int y = array[i] / BX;
        int iy, cy;
        
        for( iy = 0, cy = 0; iy < height; iy += BY, cy++ ) {
            int ix, cx;
            
            for( ix = 0, cx = 0; ix < width; ix += BX, cx++ ) {
                int xx = ix + (SKEWX(cx, cy) + x*((cx%(BX-1))+1))%BX;
                int yy = iy + (SKEWY(cx, cy) + y*((cy%(BY-1))+1))%BY;
                theRect.origin = NSMakePoint(xx-1, yy-1);
                [NSBezierPath fillRect:theRect];
            }
        }
        
        if( (BX-1) == (i%BX) ) {
            flag = true;
        }
    }
    
# undef SKEWX
# undef SKEWY
    
    if( i < SIZE )
        return true;
    
    if( (int *)0 != skews_x )
        free(skews_x);
    if( (int *)0 != skews_y )
        free(skews_y);
    
    running = false;
    return false;
    
# undef BX
# undef BY
# undef SIZE
}


#undef MAX
#undef MIN
#define MAX(a,b) ((a)>(b)?(a):(b))
#define MIN(a,b) ((a)<(b)?(a):(b))

/* from David Bagley <bagleyd@tux.org> */
static BOOL random_squares( int width, int height )
{
    static int randsize;
    static int max;
    static int *squares = 0;
    static int i;
    int columns = width / randsize + 1;  /* Add an extra for roundoff */
    NSRect rect;
    int j;
    
    [[NSColor blackColor] set];
    
    if( squares == 0 ) {
        randsize = MAX( 1, MIN(width, height) / (16 + SSRandomIntBetween(0, 31)) );
        max = (height / randsize + 1) * (width / randsize + 1);
        squares = (int *) calloc(max, sizeof (*squares));
        
        for (i = 0; i < max; i++)
            squares[i] = i;

        for (i = 0; i < max; i++) {
            int t, r;
            t = squares[i];
            r = random() % max;
            squares[i] = squares[r];
            squares[r] = t;
        }
        i = 0;
    }
    
    j = 0;
    while( i < max && j < 10 ) {
        rect.size = NSMakeSize( randsize, randsize );
        rect.origin = NSMakePoint((squares[i] % columns) * randsize, (squares[i] / columns) * randsize);
        [NSBezierPath fillRect:rect];
    
        i ++;
        j ++;
    }
    
    if( i == max ) {
        rect.size = NSMakeSize( width, height );
        rect.origin = NSMakePoint(0, 0);
        free(squares);
        squares = 0;
        [NSBezierPath fillRect:rect];
        return false;
    }
    
    return true;
}


/* idea from Rick Campbell <rick@campbellcentral.org> */
/* completely rewritten to use Bezier paths */
static BOOL spiral ( int width, int height )
{
// # define SPIRAL_START ( M_PI / 12 )
// # define SPIRAL_ANGLE_INCREMENT ( M_PI / 6 )
# define SPIRAL_START ( M_PI / 24 )
# define SPIRAL_ANGLE_INCREMENT ( M_PI / 12 )
# define SPIRAL_RADIUS_START 5 
# define SPIRAL_RADIUS_INCREMENT 2 
// note: the angle -MUST NEVER- be equal to 0, M_PI/2, M_PI, etc.

    static double angle = 0;
    double angle_new;
    static double radius;
    double radius_new;
    static BOOL flip;
    NSPoint center;
    NSPoint from;
    NSPoint middle;
    NSPoint to;
    NSBezierPath *theArc;
    double a, b, c, d;
    
    if( angle == 0 ) {
        angle = SPIRAL_START + SPIRAL_ANGLE_INCREMENT * SSRandomIntBetween(0, 11);
        flip = SSRandomIntBetween(0, 1);
        radius = SPIRAL_RADIUS_START;
    }
    
    radius_new = radius + SPIRAL_RADIUS_INCREMENT;
    if( flip )
        angle_new = angle - SPIRAL_ANGLE_INCREMENT;
    else
        angle_new = angle + SPIRAL_ANGLE_INCREMENT;
    
    [[NSColor blackColor] set];
    center = NSMakePoint(width/2, height/2);
    
    from = NSMakePoint(width/2 + radius * cos(angle),
                       height/2  + radius * sin(angle) );
    to = NSMakePoint(width/2 + radius_new * cos(angle_new),
                     height/2  + radius_new * sin(angle_new) );
    
    a = -1/tan(angle);
    b = radius/sin(angle);
    c = -1 / tan(angle_new);
    d = radius_new/sin(angle_new);
    
    middle.x = width/2 + (d-b)/(a-c);
    middle.y = height/2  + (b*c - d*a)/(c-a);
    
    theArc = [NSBezierPath bezierPath];
    [theArc moveToPoint:center];
    [theArc lineToPoint:from];
    [theArc curveToPoint:to controlPoint1:middle controlPoint2:middle];
    [theArc closePath];
    [theArc setLineWidth:2.0];
    [theArc fill];
    [theArc stroke];
    
    if( radius >= sqrt(width*width/4 + height*height/4) ) {
        NSRect rect;
        rect.size = NSMakeSize( width, height );
        rect.origin = NSMakePoint(0, 0);
        [NSBezierPath fillRect:rect];
        angle = 0;
        return false;
    }
    
    radius = radius_new;
    angle = angle_new;
    
    return true;
       
# undef SPIRAL_START
# undef SPIRAL_ANGLE_INCREMENT
# undef SPIRAL_RADIUS_START
# undef SPIRAL_RADIUS_INCREMENT
}


@end
