//
//  TestEffects.m
//  ScreenSaverTutorial
//
//  Created by pecos on Wed May 30 2001.
//  Copyright (c) 2001 __CompanyName__. All rights reserved.
//

#import "TestEffects.h"
#include <stdlib.h>

@implementation TestEffects

- (id)initWithFrame:(NSRect)frameRect isPreview:(BOOL) preview
{
    if (![super initWithFrame:frameRect isPreview:preview]) return nil;
	
    // Do your subclass initialization here
    isErasing = false;
    return self;
}

- (void)startAnimation;
{
    [super startAnimation];
}

- (void)animateOneFrame;
{
    NSColor *theColor;
    NSRect rect;
    NSSize size;
    HSVcolor startColor, middleColor, endColor;
    int i;
    
    if( isErasing == true ) {
        if( targetTime > time(0) )
            return;
        isErasing = [self eraseScreen];
        return;
    }
    
    targetTime = time(0) + 3; // 10 seconds delay
    isErasing = true;
    
    size = [self bounds].size;
    
    ncolors = NCOLORSMAX;
    
    i = SSRandomIntBetween( 0, 6 );
    switch (i) {
        case 0:
            NSLog ( @"make_smooth_colormap, %d colors of %d", ncolors, NCOLORSMAX );
            make_smooth_colormap ( colors, &ncolors );
            break;
        case 1:
            NSLog ( @"make_uniform_colormap, %d colors of %d", ncolors, NCOLORSMAX );
            make_uniform_colormap ( colors, &ncolors );
            break;
        case 2:
            NSLog ( @"make_random_colormap (bright), %d colors of %d", ncolors, NCOLORSMAX );
            make_random_colormap ( colors, &ncolors, true );
            break;
        case 3:
            NSLog ( @"make_random_colormap (not bright), %d colors of %d", ncolors, NCOLORSMAX );
            make_random_colormap ( colors, &ncolors, false );
            break;
        case 4:
            startColor.hue = SSRandomFloatBetween( 0, 1 );
            startColor.saturation = SSRandomFloatBetween( 0, 1 );
            startColor.brightness = SSRandomFloatBetween( 0, 1 );
            endColor.hue = SSRandomFloatBetween( 0, 1 );
            endColor.saturation = SSRandomFloatBetween( 0, 1 );
            endColor.brightness = SSRandomFloatBetween( 0, 1 );
            NSLog ( @"make_color_ramp (closed), %d colors of %d", ncolors, NCOLORSMAX );
            NSLog ( @" start (HSV)  %f %f %f", startColor.hue, startColor.saturation, startColor.brightness );
            NSLog ( @" end (HSV)    %f %f %f", endColor.hue, endColor.saturation, endColor.brightness );
            
            make_color_ramp (startColor, endColor, colors, &ncolors, true);
            break;
        case 5:
            startColor.hue = SSRandomFloatBetween( 0, 1 );
            startColor.saturation = SSRandomFloatBetween( 0, 1 );
            startColor.brightness = SSRandomFloatBetween( 0, 1 );
            endColor.hue = SSRandomFloatBetween( 0, 1 );
            endColor.saturation = SSRandomFloatBetween( 0, 1 );
            endColor.brightness = SSRandomFloatBetween( 0, 1 );
            NSLog ( @"make_color_ramp (not closed), %d colors of %d", ncolors, NCOLORSMAX );
            NSLog ( @" start (HSV)  %f %f %f", startColor.hue, startColor.saturation, startColor.brightness );
            NSLog ( @" end (HSV)    %f %f %f", endColor.hue, endColor.saturation, endColor.brightness );
            
            make_color_ramp (startColor, endColor, colors, &ncolors, false);
            break;
        case 6:
        default:
            startColor.hue = SSRandomFloatBetween( 0, 1 );
            startColor.saturation = SSRandomFloatBetween( 0, 1 );
            startColor.brightness = SSRandomFloatBetween( 0, 1 );
            middleColor.hue = SSRandomFloatBetween( 0, 1 );
            middleColor.saturation = SSRandomFloatBetween( 0, 1 );
            middleColor.brightness = SSRandomFloatBetween( 0, 1 );
            endColor.hue = SSRandomFloatBetween( 0, 1 );
            endColor.saturation = SSRandomFloatBetween( 0, 1 );
            endColor.brightness = SSRandomFloatBetween( 0, 1 );
            NSLog ( @"make_color_loop, %d colors of %d", ncolors, NCOLORSMAX );
            NSLog ( @" start (HSV)  %f %f %f", startColor.hue, startColor.saturation, startColor.brightness );
            NSLog ( @" middle (HSV) %f %f %f", middleColor.hue, middleColor.saturation, middleColor.brightness );
            NSLog ( @" end (HSV)    %f %f %f", endColor.hue, endColor.saturation, endColor.brightness );
            
            make_color_loop (startColor, middleColor, endColor, colors, &ncolors);
            break;
    }

    rect.size = NSMakeSize( size.width/ncolors, size.height );
    
    for( i=0; i<ncolors; i++ ) {
        theColor = [NSColor colorWithCalibratedHue: colors[i].hue
                                        saturation: colors[i].saturation
                                        brightness: colors[i].brightness
                                        alpha: 1.0 ];
        [theColor set];
        
        rect.origin = NSMakePoint( size.width/ncolors*i, 0 );
        [[NSBezierPath bezierPathWithRect:rect] fill];
        [[NSBezierPath bezierPathWithRect:rect] stroke];
    }
}


@end
